## @file
# Analysis tools. This module provides a set of general tools that are used by
# the rest of the modules of the analysis package.

#========================================================================
use vars qw($pi $deg2rad $rad2deg);
$pi=3.1415926;
$deg2rad=(2.*$pi)/360.;
$rad2deg=360./(2.*$pi);

## @method float ra($source_name)
# Returns the RA of a given source through a call to NED in units of deg
# @param source_name Source Name
# @return Source RA

#========================================================================
sub ra(){
# Makes a calle to NED
    my $source_name = $_[0];
    
    @out  = split /\s/,`$ned_coord_directory$ned_coord_script $source_name`;
    if ($out[0] eq "Target") {
	$ra = 0.;
    } else {
	$ra = $out[0];
    } 
    return($ra);
}

## @method float dec($source_name)
# Returns the DEC of a given source through a call to NED in units of deg
# @param source_name Source Name
# @return Source RA

#========================================================================
sub dec(){
# Makes a calle to NED
    my $source_name = $_[0];
    
    @out = split /\s/,`$ned_coord_directory$ned_coord_script $source_name`;
    if ($out[0] eq "Target") {
	$dec = 0.;
    } else {
	$dec = $out[1];
    }
    return($dec);
}  

## @method float Angular_separation($ra1, $ra2, $dec1,$dec2)
# Computes the angular separation between two sources in units of deg
# @param ra1 RA of Source 1
# @param ra2 RA of Source 2
# @param dec1 DEC of Source 1
# @param dec2 DEC of Source 2
# @return Source Angular Separation

#========================================================================
sub Angular_separation(){

    my $ra1  = $_[0];
    my $ra2  = $_[1];
    my $dec1 = $_[2];
    my $dec2 = $_[3];

    my $delta_alpha = ($ra2-$ra1);
    my $num = sqrt(
                   (cos($dec2*$deg2rad)*sin($delta_alpha*$deg2rad))**2
                   +(cos($dec1*$deg2rad)*sin($dec2*$deg2rad)
                     -sin($dec1*$deg2rad)*cos($dec2*$deg2rad)*cos($delta_alpha*$deg2rad))**2
                   );
    my $den = sin($dec1*$deg2rad)*sin($dec2*$deg2rad)
        +cos($dec1*$deg2rad)*cos($dec2*$deg2rad)*cos($delta_alpha*$deg2rad);
    if ($den != 0.) {return($rad2deg*atan2($num,$den));}
    return(1000.);
}

## @method float nh($ra, $dec)
# Computes the galactic Nh in untis of cm-2
# @param ra RA of Source
# @param dec DEC of Source
# @return Source Galactic Nh

#========================================================================
sub nh(){
    my $ra  = $_[0];
    my $dec = $_[1];

    chomp(my $nh = `nh map=/usr/local/heasoft6.0/i686-pc-linux-gnu-libc2.3/refdata/h1_nh.fits equinox=2000 ra=$ra dec=$dec | grep average`);
    ($temp, $n_h) = split/\)/,$nh;
    $n_h =~ s/\s+//;
    print "\n   \#> Galactic NH in units of cm-2 \n"; 
    print "       RA : $ra \n";
    print "       DEC: $dec \n";
    print "       NH : $n_h \n";
    if ($Log_Flag == 0) {
	print MYOUTFILE "\n   \#> Galactic NH in units of cm-2 \n"; 
	print MYOUTFILE "       RA : $ra \n";
	print MYOUTFILE "       DEC: $dec \n";
	print MYOUTFILE "       NH : $n_h \n";
    }
    if ($Log_Spec_Flag == 0) {
	print SPECTRALOG "\n   \#> Galactic NH in units of cm-2 \n"; 
	print SPECTRALOG "       RA : $ra \n";
	print SPECTRALOG "       DEC: $dec \n";
	print SPECTRALOG "       NH : $n_h \n";
    }

    $gv->SRC_NH($nh); # Store nh

    return($n_h);
}

## @method void addsourcetoregionfile($region_file,$x,$y,$R1, $R2, $color, $text, $region_type, $coord_type  )
# Creates a region file that can be read by ds9*
# (http://www.physics.louisville.edu/help/ds9/region.html)
# @param region_file Region File Name
# @param x X-coord of the center of region
# @param y Y-coord of the center of region
# @param R1 radius of inner Circle / X-size of Box
# @param R2 radius of outer Circle(annulus) / Y-size of Box
# @param color Color of region
# @param text Text to be assigned to the region
# @param region_type Region Type (Regions Supported: Circle, Annulus, Box)
# @param coord_type Type of Coordinates (FK5, IMAGE, PHYSICAL, ....)
# @return Void

#========================================================================
sub addsourcetoregionfile(){

    my $region_file = $_[0];
    my $x           = $_[1];
    my $y           = $_[2];
    my $R1          = $_[3];
    my $R2          = $_[4];
    my $color       = $_[5];
    my $text        = $_[6];
    my $region_type = $_[7];
    my $coord_type  = $_[8];
	
    open (REGIONFILE, ">>$region_file") or die "couldn't open file $region_file: $! \n";

    if ($region_type eq "circle") {print REGIONFILE "$coord_type\;circle($x,$y,$R2) \# color=$color text=\"$text\" \n";}
    if ($region_type eq "annulus") {print REGIONFILE "$coord_type\;annulus($x,$y,$R1,$R2) \# color=$color text=\"$text\" \n";}
    if ($region_type eq "box") {print REGIONFILE "$coord_type\;box($x,$y,$R1,$R2) \# color=$color text=\"$text\" \n";}

    close(REGIONFILE);
    return(1);
}

## @method void produce_ps_file($fits_file,$region_file,$title,$ps_file)
# Creates a ps image file from a fits image file (call to ds9)
# @param fits_file Fits File Name
# @param region_file Region File Name
# @param title Title ofImage
# @param ps_file Postscript File Name
# @return Void

#========================================================================
sub produce_ps_file(){

    my $fits_file    = $_[0];
    my $region_file  = $_[1];
    my $title        = $_[2];
    my $ps_file      = $_[3];

    my @args = ("ds9 -cmap invert yes -cmap b -cmap value 1.15 .13 -fits $fits_file -region $region_file -zoom to fit -title $title -print destination file -print filename $ps_file -print palette rgb -print level 1 -print resolution 300 -print -exit");
    system(@args);                   # == 0 or &error_code(5); NOTE: If I put this bit it wont run on the grid    
    return(1);

#Another command to produce images
#implot set=pn_img_gtifiltered_smooth.img colourmap=4 device=/PS srclisttab=../images/pn_emllist.fits.gz expression='(ID_BAND==0)' -i ccf.cif gridstyle='grid' withsrclisttab=y srccolour=4 radiusstyle='user' userradius=200.

}


sub deg2hour()
{
	$deg = $_[0];
	
	$hour = $deg*0.0666667;
	$HH = int($hour);
	
	$rest = ($hour - int($hour)) * 60.;
	$MM = int($rest);
	$SS = ($rest - $MM)*60.;
	 
	return("$HH:$MM:$SS");

}

sub deg2sexa()
{
	$degrees = $_[0];
	$neg = 0;
	if ($degrees < 0) {
		$neg = 1;
		$degrees = -$degrees;
	}
	$deg = int($degrees);
	
	$min = int(($degrees - $deg)*60.);
	$sec = (($degrees - $deg)*60. - $min)*60.;
	if ($neg == 1) { $deg = -$deg; }
	
	return("$deg:$min:$sec");
	
}
## @sub event_file_spectralinf($event_file, $instr)
# Extract information from an spectral event file
# @param event_file Event File Name
# @param instr Instrument
# @return Live Time, Background Scale Factor, Area Scale factor

#========================================================================
sub event_file_spectralinf(){

    my $event_file = $_[0];
    my $instr      = $_[1];

#.. Print information from fits spectral file
#.. NOTE: In this type of file EXPOSURE=Weighted live time of CCDs in extraction region

    chomp(my $live_time      = `fkeyprint infile=$event_file+1 keynam=EXPOSURE outfil=STDOUT exact=yes | grep EXPOSURE | tail -1 | cut -c11-30`);
    $live_time =~ s/\s+//;
    $live_time =~ s/\'+//;
    if ($live_time eq "EXPOSURE" || $live_time eq "" || $live_time == 0) {$live_time ="unknown";}

    $backsale = &getbackscale($event_file,1);
    $areascal = &getareascale($event_file,1);

    print "   \#> Information from Spectra File: $event_file \n";
    print "       $instr Live Time : $live_time (Weighted live time of CCDs in extraction region) \n";   
    print "       $instr Backscale : $backsale   \n";    
    print "       $instr Areascale : $areascal   \n";      
    if ($Log_Flag == 0) {
	print MYOUTFILE "   \#> Information from Event File: $event_file \n";
	print MYOUTFILE "       $instr Live Time : $live_time (Weighted live time of CCDs in extraction region) \n";   
	print MYOUTFILE "       $instr Backscale : $backsale   \n";    
	print MYOUTFILE "       $instr Areascale : $areascal   \n";   
    }
    if ($Log_Spec_Flag == 0) {
	print SPECTRALOG "   \#> Information from Spectra File: $event_file \n";
	print SPECTRALOG "       $instr Live Time : $live_time (Weighted live time of CCDs in extraction region) \n";   
	print SPECTRALOG "       $instr Backscale : $backsale   \n";    
	print SPECTRALOG "       $instr Areascale : $areascal   \n";      
    }
    if ($Log_Lcur_Flag == 0) {
	print LCURVELOG "   \#> Information from Spectra File: $event_file \n";
	print LCURVELOG "       $instr Live Time : $live_time (Weighted live time of CCDs in extraction region) \n";   
	print LCURVELOG "       $instr Backscale : $backsale   \n";    
	print LCURVELOG "       $instr Areascale : $areascal   \n";      
    }
    return($live_time,$backsale,$areascal);
}

## @sub event_file_inf($event_file, $instr)
# Extract information from an event file
# @param event_file Event File Name
# @param instr Instrument
# @return Mode, Filter, Submode, UTC Date, Duration, ON Time,Live Time, Obs Start, Obs End

#========================================================================
sub event_file_inf(){
#.. Print fits file information

    my $event_file = $_[0];
    my $instr      = $_[1];
    
    $ext = 1;
    if ($instr eq "om"){$ext = 0;}

    chomp(my $rows   = `fstruct $event_file+$ext colinfo=no | grep BINTABLE | awk \'\{print \$6\}\'`);
    $rows     =~ s/\s+//;
    $rows     =~ s/\'+//;
    if ($rows eq "") {$rows ="unknown";}

    chomp(my $tstart   = `fdump $event_file+$ext outfile=STDOUT columns=TIME ROWS=1 prhead=no showcol=no prdata=yes showunit=no showrow=no showscale=no`);
    $tstart     =~ s/\s+//;
    $tstart     =~ s/\'+//;
    if ($tstart eq "") {$tstart ="unknown";}

    chomp(my $tend   = `fdump $event_file+$ext outfile=STDOUT columns=TIME ROWS=$rows prhead=no showcol=no prdata=yes showunit=no showrow=no showscale=no`);
    $tend     =~ s/\s+//;
    $tend     =~ s/\'+//;
    if ($tend eq "") {$tend ="unknown";}

    $dmode        = &getdatamode($event_file,$ext);
    $mode         = &getsubmode($event_file,$ext);
    $filter       = &pse_getfilter($event_file,$ext);
    $utc_obsdate  = &getdate_obs($event_file,$ext);
    $obs_duration = &getduration($event_file,$ext);
    $ontime       = &getontime($event_file,$ext);
    $livetime     = &getlivetime($event_file,$ext);
    $exposure     = &getexposure($event_file,$ext);
 
    print "   \#> Information from Event File: $event_file \n";
    print "       $instr OBS DATE          : $utc_obsdate  \n";
    print "       $instr OBS STARTING TIME : $tstart (1st Event)\n";
    print "       $instr OBS ENDING TIME   : $tend (Last Event)\n";
    print "       $instr EXPOSURE ID  : $exposure \n";
    print "       $instr OBS DURATION : $obs_duration   \n";
    print "       $instr OBS ONTIME   : $ontime   \n";
    print "       $instr OBS LIVETIME : $livetime  \n";
    print "       $instr D MODE       : $dmode  \n";
    print "       $instr SUBMODE      : $mode   \n";
    print "       $instr FILTER       : $filter \n";	  
    if ($Log_Flag == 0) {
	print MYOUTFILE "   \#> Information from Event File: $event_file \n";
	print MYOUTFILE "       $instr OBS DATE          : $utc_obsdate  \n";
	print MYOUTFILE "       $instr OBS STARTING TIME : $tstart (1st Event)\n";
	print MYOUTFILE "       $instr OBS ENDING TIME   : $tend (Last Event)\n";
	print MYOUTFILE "       $instr EXPOSURE ID  : $exposure \n";
	print MYOUTFILE "       $instr OBS DURATION : $obs_duration   \n";
	print MYOUTFILE "       $instr OBS ONTIME   : $ontime   \n";
	print MYOUTFILE "       $instr OBS LIVETIME : $livetime  \n";
	print MYOUTFILE "       $instr D MODE       : $dmode  \n";
	print MYOUTFILE "       $instr SUBMODE      : $mode   \n";
	print MYOUTFILE "       $instr FILTER       : $filter \n";
    }
    if ($Log_Evt_Flag == 0) {
	print EVTLOG "   \#> Information from Event File: $event_file \n";
	print EVTLOG "       $instr OBS DATE          : $utc_obsdate  \n";
	print EVTLOG "       $instr OBS STARTING TIME : $tstart  (1st Event)\n";
	print EVTLOG "       $instr OBS ENDING TIME   : $tend (Last Event)\n";
	print EVTLOG "       $instr EXPOSURE ID  : $exposure \n";
	print EVTLOG "       $instr OBS DURATION : $obs_duration   \n";
	print EVTLOG "       $instr OBS ONTIME   : $ontime   \n";
	print EVTLOG "       $instr OBS LIVETIME : $livetime  \n";
	print EVTLOG "       $instr D MODE       : $dmode  \n";
	print EVTLOG "       $instr SUBMODE      : $mode   \n";
	print EVTLOG "       $instr FILTER       : $filter \n";
    }

    &pse_fill_structure($gv->SRC_OBSID,$instr,'date',$utc_obsdate);
    &pse_fill_structure($gv->SRC_OBSID,$instr,'starting_time',$tstart);
    &pse_fill_structure($gv->SRC_OBSID,$instr,'ending_time',$tend);
    &pse_fill_structure($gv->SRC_OBSID,$instr,'duration',$obs_duration);
    &pse_fill_structure($gv->SRC_OBSID,$instr,'ontime',$ontime);
    &pse_fill_structure($gv->SRC_OBSID,$instr,'livetime',$livetime);
    &pse_fill_structure($gv->SRC_OBSID,$instr,'mode',$dmode);
    &pse_fill_structure($gv->SRC_OBSID,$instr,'submode',$mode);
    &pse_fill_structure($gv->SRC_OBSID,$instr,'filter',$filter);
    &pse_fill_structure($gv->SRC_OBSID,$instr,'exposure',$exposure);
 
    return($mode,$filter,$dmode,$utc_obsdate,$obs_duration,$ontime,$livetime,$tstart,$tend);
}

## @sub setcoordinates($event_file)
# Check if we have to set the coordinates
# @return 

#========================================================================
sub setcoordinates(){
	#.. Test for odfingest file and set ENV variable     
    @test_odfingest=&test_odfingest();
    my $odfingestfile=$test_odfingest[1];
    if (!$test_odfingest[0]) {	
	&run_odfingest();
    }
    @test_odfingest=&test_odfingest();
    if (!$test_odfingest[0]) {print "      Run odfingest first \n"; &error_code(255);}
    &set_odfingest($odfingestfile);
    
	
	if (($gv->SRC_RA != -999) && ($gv->SRC_DEC != -999))
		{
			print "   \#> Using user defined coordinates\n";
			print MYOUTFILE "   \#> Using user defined coordinates\n";
			$ra = $gv->SRC_RA;
			$dec = $gv->SRC_DEC;
			print MYOUTFILE "   \#> RA_OBJ = $ra \n";
			print "   \#> RA_OBJ = $ra\n";
			print MYOUTFILE "   \#> DEC_OBJ = $dec \n";
			print "   \#> DE_OBJ = $dec\n";
		
		}
	elsif (($gv->SRC_RA == -999) && ($gv->SRC_DEC == -999))
		{
			print "   \#> Not user defined coordinates found. Using the SUM.SAS RA_OBJ and DE_OBJ coordinates.\n";
			print MYOUTFILE "   \#> Not user defined coordinates found. Using the SUM.SAS	 RA_OBJ and DE_OBJ coordinates.\n";
			my $ra = &getobsra($odfingestfile);
			my $dec =	&getobsdec($odfingestfile);
			$gv->SRC_RA($ra);
			$gv->SRC_DEC($dec);
			print MYOUTFILE "   \#> RA_OBJ = $ra\n";
			print "   \#> RA_OBJ = $ra\n";
			print MYOUTFILE "   \#> DEC_OBJ = $dec\n";
			print "   \#> DE_OBJ = $dec\n";
			
		} 
	else
		{
			print "   \#> One of the source coordinates is not set. Please, check your input parameters \n";
			print MYOUTFILE "   \#> One of the source coordinates is not set. Please, check your input parameters \n";
			&error_code(102) && return;
		}
	return(1);
}

## @sub getobsra($event_file)
# Get Observation RA
# @param event_file Event File Name
# @return Observation RA

#========================================================================
sub getobsra(){
    my $event_file = $_[0]; 
    
    chomp(my $obj_ra   = `grep "Target Right Ascension" $event_file  | awk '{print \$1} '`);
    $obj_ra     =~ s/\s+//;    
    $obj_ra = ($obj_ra*15.);
    return($obj_ra);
}

## @sub getobsdec($event_file)
# Get Observation DEC
# @param event_file Event File Name
# @return Observation DEC

#========================================================================
sub getobsdec(){
    my $event_file = $_[0]; 
    
    chomp(my $obj_dec   = `grep "Target Declination" $event_file  | awk '{print \$1} '`);
    $obj_dec     =~ s/\s+//;    
    
    return($obj_dec);
}

## @sub getdatamode($event_file,$ext)
# Get Exposure Mode
# @param event_file Event File Name
# @param ext Extension of FITS file
# @return Exposure Mode

#========================================================================
sub getdatamode(){
    my $event_file = $_[0]; 
    my $ext        = $_[1]; 
    chomp(my $dmode   = `fkeyprint infile=$event_file+$ext keynam=DATAMODE outfil=STDOUT exact=yes | grep DATAMODE | tail -1 | cut -c12-20`);
    $dmode     =~ s/\s+//;
    $dmode     =~ s/\'+//;
    if ($dmode eq "DATAMODE" || $dmode eq "") {$dmode ="unknown";}
    return($dmode);
}

## @sub getsubmode($event_file,$ext)
# Get Exposure Submode
# @param event_file Event File Name
# @param ext Extension of FITS file
# @return Exposure Submode

#========================================================================
sub getsubmode(){
    my $event_file = $_[0]; 
    my $ext        = $_[1]; 
    chomp(my $mode   = `fkeyprint infile=$event_file+$ext keynam=SUBMODE outfil=STDOUT exact=yes | grep SUBMODE | tail -1 | cut -c12-28`);
 	$mode =~ s/^\s+//;
	$mode =~ s/\s+$//;
    $mode =~ s/^\'+//;
    $mode =~ s/\'+$//;
    if ($mode eq "SUBMODE" || $mode eq "") {$mode ="unknown";}
    
    return($mode);
}

## @sub pse_getfilter($event_file,$ext)
# Get Exposure Filter
# @param event_file Event File Name
# @param ext Extension of FITS file
# @return Exposure Filter

#========================================================================
sub pse_getfilter(){
    my $event_file = $_[0]; 
    my $ext        = $_[1];  
    chomp(my $filter = `fkeyprint infile=$event_file+$ext keynam=FILTER outfil=STDOUT exact=yes | grep FILTER | tail -1 | cut -c12-25`);
    $filter    =~ s/\s+\'+//;
    $filter    =~ s/\'+//;
    if ($filter eq "FILTER" || $filter eq "" || $filter eq "UNKNOWN") {$filter ="unknown";}
    return($filter);
}

## @sub getdate_obs($event_file,$ext)
# Get Observation Date
# @param event_file Event File Name
# @param ext Extension of FITS file
# @return Observation Date

#========================================================================
sub getdate_obs(){
    my $event_file = $_[0]; 
    my $ext        = $_[1];  
    chomp($utc_obsdate   = `fkeyprint infile=$event_file+$ext keynam=DATE_OBS outfil=STDOUT exact=yes | grep DATE_OBS | tail -1 | cut -c12-34`);
    if ($utc_obsdate eq "" || $utc_obsdate eq "DATE_OBS") 
    {chomp($utc_obsdate     = `fkeyprint infile=$event_file+$ext keynam=DATE-OBS outfil=STDOUT exact=yes | grep DATE-OBS | tail -1 | cut -c12-34`);}
    $utc_obsdate  =~ s/\s+//;
    $utc_obsdate  =~ s/(\'\/)//;
    if ($utc_obsdate eq "DATE-OBS" || $utc_obsdate eq ""){$utc_obsdate="unknown";}
    return($utc_obsdate);
}

## @sub getduration($event_file,$ext)
# Get Observation Duration Time in seconds
# @param event_file Event File Name
# @param ext Extension of FITS file
# @return Observation Duration Time in seconds

#========================================================================
sub getduration(){
    my $event_file = $_[0]; 
    my $ext        = $_[1];   
    chomp(my $obs_duration  = `fkeyprint infile=$event_file+$ext keynam=DURATION outfil=STDOUT exact=yes | grep DURATION | tail -1 | cut -c11-30`);
    $obs_duration =~ s/\s+//;
    if ($obs_duration eq "" || $obs_duration eq "DURATION") 
    {chomp($obs_duration  = `fkeyprint infile=$event_file+$ext keynam=TELAPSE outfil=STDOUT exact=yes | grep TELAPSE | tail -1 | cut -c11-30`);}
    $obs_duration =~ s/\s+//;
    if ($obs_duration eq "TELAPSE" || $obs_duration eq "" || $obs_duration == 0){$obs_duration="unknown";}
    return($obs_duration);
}

## @sub getontime($event_file,$ext)
# Get Observation ON Time in seconds
# @param event_file Event File Name
# @param ext Extension of FITS file
# @return Observation ON Time in seconds

#========================================================================
sub getontime(){
    my $event_file = $_[0]; 
    my $ext        = $_[1]; 
    chomp(my $ontime        = `fkeyprint infile=$event_file+$ext keynam=ONTIME outfil=STDOUT exact=yes | grep ONTIME | tail -1 | cut -c11-30`);
    $ontime    =~ s/\s+//;
    if ($ontime eq "ONTIME" || $ontime eq "" || $ontime == 0){$ontime="unknown";}
    return($ontime);
}

## @sub getlivetime($event_file,$ext)
# Get Observation Live Time in seconds
# @param event_file Event File Name
# @param ext Extension of FITS file
# @return Observation Live Time in seconds

#========================================================================
sub getlivetime(){
    my $event_file = $_[0]; 
    my $ext        = $_[1];     
    chomp(my $livetime      = `fkeyprint infile=$event_file+$ext keynam=LIVETIME outfil=STDOUT exact=yes | grep LIVETIME | tail -1 | cut -c11-30`);
    $livetime  =~ s/\s+//;
    if ($livetime eq "LIVETIME" || $livetime eq "" || $livetime == 0){$livetime="unknown";}
    return($livetime);
}

## @sub getexposure($event_file,$ext)
# Get Observation Exposure in seconds
# @param event_file Event File Name
# @param ext Extension of FITS file
# @return Observation Exposure in seconds

#========================================================================
sub getexposure(){
    my $event_file = $_[0]; 
    my $ext        = $_[1];
    chomp(my $exposure      = `fkeyprint infile=$event_file+$ext keynam=EXPIDSTR outfil=STDOUT exact=yes | grep EXPIDSTR | tail -1 | cut -c12-17`);
    $exposure =~ s/\s+//;
    if ($exposure eq "EXPIDSTR" || $exposure eq "EXPIDS" || $exposure eq ""){$exposure="unknown";}
    return($exposure);
} 

## @sub getinstrume($event_file,$ext)
# Get instrument
# @param event_file Event File Name
# @param ext Extension of FITS file
# @return Instrument name

#========================================================================
sub getinstrume(){
    my $event_file = $_[0]; 
    my $ext        = $_[1];
    chomp(my $inst      = `fkeyprint infile=$event_file+$ext keynam=INSTRUME outfil=STDOUT exact=yes | grep INSTRUME | tail -1 | cut -c12-17`);
    $inst =~ s/\s+//;
    if ($inst ne "EPN" && $inst ne "EMOS1" && $inst ne "EMOS2"){&error_code(255);}
    return($inst);
} 

## @sub getnumberofentries($event_file,$ext)
# Get Number of Entries on an Event File
# @param event_file Event File Name
# @param ext Extension of FITS file
# @return Number of Entries

#========================================================================
sub getnumberofentries(){  
    my $event_file = $_[0];
    my $ext        = $_[1];
    chomp(my $entries = `fkeyprint infile=$event_file+$ext keynam=NAXIS2 outfil=STDOUT exact=yes | grep NAXIS2 | tail -1 | cut -c12-30`);
    $entries =~ s/\s+//;
    if ($entries eq "") {$entries =-1.;}
    return($entries);	
}

## @sub getbackscale($event_file,$ext)
# Get background scaling factor
# @param event_file Event File Name
# @param ext Extension of FITS file
# @return Backscale

#========================================================================
sub getbackscale(){  
    my $event_file = $_[0];
    my $ext        = $_[1];
    chomp(my $backsale       = `fkeyprint infile=$event_file+$ext keynam=BACKSCAL outfil=STDOUT exact=yes | grep BACKSCAL | tail -1 | cut -c11-30`);
    $backsale =~ s/\s+//;
    $backsale =~ s/\'+//;
    if ($backsale eq "BACKSCAL" || $backsale eq "" || $backsale == 0) {$backsale ="unknown";}
    return($backsale);	
}

## @sub getareascale($event_file,$ext)
# Get area scaling factor
# @param event_file Event File Name
# @param ext Extension of FITS file
# @return Areascale

#========================================================================
sub getareascale(){  
    my $event_file = $_[0];
    my $ext        = $_[1];    
    chomp(my $areascal       = `fkeyprint infile=$event_file+$ext keynam=AREASCAL outfil=STDOUT exact=yes | grep AREASCAL | tail -1 | cut -c11-30`);
    $areascal =~ s/\s+//;
    $areascal =~ s/\'+//;    
    if ($areascal eq "AREASCAL" || $areascal eq "" || $areascal == 0) {$areascal ="unknown";}
    return($areascal);	
}


## @sub gti_file_inf($event_file, $instr)
# Extract information from an GTI file
# @param event_file GTI File Name
# @param instr Instrument
# @return Sum of GTI

#========================================================================
sub gti_file_inf(){

    my $event_file = $_[0];
    my $instr      = $_[1];

    chomp(my $gti_time =`fkeyprint infile=$event_file+1 keynam=ONTIME outfil=STDOUT exact=yes | grep ONTIME | tail -1 | cut -c10-30`);
    $gti_time  =~ s/\s+//;    
    print "   \#> Information from GTI File: $event_file \n";
    print "      $instr Sum of all Good Time Intervals  : $gti_time  \n";
    if ($Log_Flag == 0) {
	print MYOUTFILE "   \#> Information from GTI File: $event_file \n";
	print MYOUTFILE "      $instr Sum of all Good Time Intervals  : $gti_time  \n";	
    }
    if ($Log_GTIEvt_Flag == 0) {
	print GTILOG "   \#> Information from GTI File: $event_file \n";
	print GTILOG "      $instr Sum of all Good Time Intervals  : $gti_time  \n";
    }


    return($gti_time);	
}

## @method void produce_image($event_file,$image_file,$instr)
# Produce EPIC Images
# @param event_file Name of Event File
# @param image_file Name of Image File
# @param instr Instrument
# @return Void

#========================================================================
sub produce_image(){ 

    my $event_file = $_[0];
    my $image_file = $_[1];
    my $instr      = $_[2];

    print "   \#> Producing image from $event_file .... ";
    if ($Log_Flag == 0) {print MYOUTFILE "   \#> Producing image from $event_file .... ";}
        
    if ($instr eq "pn") {$expr="\#XMMEA_EP";}
    if ($instr eq "m1"){$expr="\#XMMEA_EM";}
    if ($instr eq "m2"){$expr="\#XMMEA_EM";}

    my $mode = $obsid_instruments{$gv->SRC_OBSID}{$instr}{"mode"};
    if ($mode eq "IMAGING" || $mode eq "imaging") { 
	$xbin = 80; 
	$ybin = 80;
	$xval = "X"; 
	$yval = "Y"; 
    } elsif (($mode eq "TIMING" || $mode eq "timing")
	     && ($instr eq "pn")) {
	$xbin = 1; 
	$ybin = 1; 
	$xval = "RAWX"; 
	$yval = "RAWY"; 
    } elsif (($mode eq "TIMING" || $mode eq "timing")
	&& ($instr eq "m1" || $instr eq "m2")) {
	$xbin = 1; 
	$ybin = 1; 
	$xval = "RAWX"; 
	$yval = "TIME"; 
    } else {
	print "ERROR \n";
	if ($Log_Flag == 0) {print MYOUTFILE "ERROR \n";}
	return(0);
    }

    @args = ("evselect table=\"$event_file\" expression=\"$expr\" withimageset=yes imageset=\"$image_file\" imagebinning=binSize ximagebinsize=$xbin yimagebinsize=$ybin xcolumn=\"$xval\" ycolumn=\"$yval\"");
    system(@args) == 0 or &error_code(6);

    if ($Log_Flag == 0) {print MYOUTFILE "OK \n";}
    
    return(1);
}

## @method void produce_smooth_image($event_file,$image_file,$smoothimage_file,$instr)
# Produce EPIC Images
# @param event_file Name of Event File
# @param image_file Name of Image File
# @param smoothimage_file Name of Smooth Image File
# @param instr Instrument
# @return Void

#========================================================================
sub produce_smooth_image(){   

    my $event_file       = $_[0];
    my $image_file       = $_[1];
    my $smoothimage_file = $_[2];
    my $instr            = $_[3];

    print "   \#> Producing smooth image from $event_file  .... \n";
    if ($Log_Flag == 0) {print MYOUTFILE "   \#> Producing smooth image from $event_file .... \n";}

    if ($instr eq "pn") {$expr="\#XMMEA_EP";}
    if ($instr eq "m1"){$expr="\#XMMEA_EM";}
    if ($instr eq "m2"){$expr="\#XMMEA_EM";}

    my $mode = $obsid_instruments{$gv->SRC_OBSID}{$instr}{"mode"};
    if ($mode eq "IMAGING" || $mode eq "imaging") { 
	$xbin = 80; 
	$ybin = 80;
	$xval = "X"; 
	$yval = "Y"; 
    } elsif (($mode eq "TIMING" || $mode eq "timing")
	     && ($instr eq "pn")) {
	$xbin = 1; 
	$ybin = 1; 
	$xval = "RAWX"; 
	$yval = "RAWY"; 
    } elsif (($mode eq "TIMING" || $mode eq "timing")
	&& ($instr eq "m1" || $instr eq "m2")) {
	$xbin = 1; 
	$ybin = 1; 
	$xval = "RAWX"; 
	$yval = "TIME"; 
    } else {
	print "ERROR \n";
	if ($Log_Flag == 0) {print MYOUTFILE "ERROR \n";}
	return(0);
    }

    @args = ("evselect table=\"$event_file\" expression=\"$expr\" withimageset=yes imageset=\"$image_file\" imagebinning=binSize ximagebinsize=$xbin yimagebinsize=$ybin xcolumn=\"$xval\" ycolumn=\"$yval\"");   	
    system(@args) == 0 or &error_code(6);  
       
    @args = ("(asmooth inset=$image_file outset=$smoothimage_file smoothstyle=simple convolverstyle=gaussian width=2.5) >& /dev/null");
    system(@args) == 0 or &error_code(6);

    print "Smooth image created. \n"; 
    if ($Log_Flag == 0) {print MYOUTFILE "OK \n";}
    
    return(1);
}

## @method void print_sas_setup()
# Prints SAS Setup parameters
# @return Void

#========================================================================
sub print_sas_setup(){
#.. Print out of SAS set-up
    print "   \#> This is the current SAS set-up: \n";
    my $sas_info = `printenv | grep SAS`;
    @args = ("printenv | grep SAS");
    system(@args) == 0 or &error_code(3);
    if ($Log_Flag == 0) {
	print MYOUTFILE "   \#> This is the current SAS set-up: \n";
	print MYOUTFILE "$sas_info";
    }
    return(1);
}   

## @method string check_sourceName($file)
# Replace any "+" character in the $fileName by "_"
# return: valid file name

#=====================================================================
sub check_sourceName()
{
	my $validName = $_[0];
	#... Replacing "+" character by "_"
	$validName =~ s/\+/\_/;
	#... Replacing " " character by "_"
	$validName =~ s/\ /\_/;
	
	return ($validName);
	
}

1;

